@@ -1,27 +1,7 @@
 /*
- * Copyright (c) 2012-2016 Ericsson AB
- * All rights reserved.
+ * Copyright (c) 2012-2018 Linutronix GmbH. All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- *
- * 1. Redistributions of source code must retain the above copyright notice,
- *    this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright notice,
- *    this list of conditions and the following disclaimer in the documentation
- *    and/or other materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
- * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
+ * SPDX-License-Identifier: BSD-2-Clause
  */
 
 #include <stdio.h>
@@ -1101,6 +1081,16 @@ static unsigned int get_tar_checksum(struct tar_header *header)
 	for (i = 0; i < BLOCK_SIZE; i++)
 		sum += 0xff & buf[i];
 
+	/*
+	 * There are only 7 characters available in the header to store the
+	 * checksum. With a block size of 512, only 6 characters are needed
+	 * for the maximal value. However, gcc does not realize this. The
+	 * final result is checked to help the compiler realize that this
+	 * function does indeed return a value that will fit in the header.
+	 * Note that this final check can never be true.
+	 */
+	if (sum > (BLOCK_SIZE * 0xff))
+		return 0;
 	return sum;
 }
 
@@ -1241,7 +1231,8 @@ static int dump_compressed_tar(struct dump_info *di)
 	snprintf(hdr.mode, sizeof(hdr.mode), "%07o", 0644);
 	snprintf(hdr.uid, sizeof(hdr.uid), "%07o", 0);
 	snprintf(hdr.gid, sizeof(hdr.gid), "%07o", 0);
-	snprintf(hdr.mtime, sizeof(hdr.mtime), "%011lo", time(NULL));
+	snprintf(hdr.mtime, sizeof(hdr.mtime), "%011llo",
+		 (long long)time(NULL));
 	memset(hdr.checksum, ' ', sizeof(hdr.checksum));
 	hdr.type = 'S';
 	memcpy(hdr.magic, "ustar ", 6);
@@ -3605,7 +3596,7 @@ out:
 
 static int do_all_dumps(struct dump_info *di, int argc, char *argv[])
 {
-	struct config *cfg;
+	struct config *cfg = NULL;
 	const char *recept;
 	bool live_dumper;
 	char *comm_base;
